// -*- C++ -*-
//
//  This file is part of CBNSIM, the Content-Based Networking
//  Simulation Library.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: SendMessageWNP.cc,v 1.3 2004/04/19 05:26:16 rutherfo Exp $
//
#include <exception>
#include <siena/ssbg.h>
#include <siena/ssim.h>
#include <stdexcept>
#include "cbnsim/SendMessageWNP.h"
#include "cbnsim/logging.h"

using namespace cbnsim;
using namespace std;

SendMessageWNP :: SendMessageWNP( unsigned int nid,
				  ostream& out,
				  ssbg::SSBG& ssbg,
				  unsigned int delay,
				  bool isconst)
    : WorkloadNodeProcess( nid, out ),
      m_delay( delay ), m_ssbg( ssbg ), m_const (isconst)
{
    if( m_const )
    {
	m_ssbg.new_notification( m_notif );
    }
	
}

void SendMessageWNP :: init()
{
    if( m_delay )
    {
	ssim::Sim::self_signal_event( NULL, ssbg::rand_poisson_delta( m_delay ) );
    }
}

void SendMessageWNP :: print( const ssbg::notification& notif )
{
    ssbg::notification::const_iterator ni;
    m_out << "send_message " << node_id() << " " << ssim::Sim::clock();
    for( ni = notif.begin(); ni != notif.end(); ++ni )
    {
	m_out << " ";

	switch( (*ni).type )
	{
	case ssbg::BOOL:
	    output_attr( m_out, (*ni).name, "=", (*ni).bool_val );
	    break;
	case ssbg::DOUBLE:
	    output_attr( m_out, (*ni).name, "=", (*ni).double_val );
	    break;
	case ssbg::INT:
	    output_attr( m_out, (*ni).name, "=", (*ni).int_val );
	    break;
	case ssbg::STRING:
	    output_attr( m_out, (*ni).name, "=", (*ni).string_val );
	    break;
	default:
	    throw logic_error( "unknown attribute type" );
	}
    }
    m_out << " ;" << endl;
}

void SendMessageWNP :: process_event( const ssim::Event* evt )
{
  CBNASSERT( !evt, "SendMessageWNP can only handle timeout events" );
  if(!m_const)
    {
      m_ssbg.new_notification( m_notif );
    }
  
  print( m_notif );
  ssim::Sim::self_signal_event( NULL, ssbg::rand_poisson_delta( m_delay ) );
}
