// -*- C++ -*-
//
//  This file is part of CBNSIM, the Content-Based Networking
//  Simulation Library.
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: SetPredicateWNP.cc,v 1.5 2004/04/19 05:26:16 rutherfo Exp $
//
#include <exception>
#include <siena/ssbg.h>
#include <siena/ssim.h>
#include <stdexcept>
#include "cbnsim/SetPredicateWNP.h"
#include "cbnsim/logging.h"

using namespace cbnsim;
using namespace std;

SetPredicateWNP :: SetPredicateWNP( unsigned int nid,
				    ostream& out,
				    ssbg::SSBG& ssbg,
				    unsigned int delay,
				    bool t0_pred,
				    unsigned int msg_rate)
    : WorkloadNodeProcess( nid, out ),
      m_delay( delay ), m_ssbg( ssbg ), m_t0_pred( t0_pred ),
      m_msg_rate(msg_rate)
{
}

void SetPredicateWNP :: init()
{
    if( m_t0_pred )
    {
	ssbg::predicate pred;
	m_ssbg.new_predicate( pred );
	print( pred );
    }
    if( m_delay )
    {
	ssim::Sim::self_signal_event( NULL, ssbg::rand_poisson_delta( m_delay ) );
    }
}

void SetPredicateWNP :: print( const ssbg::predicate& pred )
{
    ssbg::predicate::const_iterator pi;
    
    unsigned int rate = ssbg::rand_poisson_delta( m_msg_rate );
	
    m_out << "set_predicate " << node_id() << " " << ssim::Sim::clock() << " "
	  << (m_msg_rate ? rate : 0) << " ";
    for( pi = pred.begin(); pi != pred.end(); ++pi )
    {
	if( (*pi).empty() )
	{
	    continue;
	}
	if( pi != pred.begin() )
	{
	    m_out << " | ";
	}
	ssbg::filter::const_iterator j;
	for( j = (*pi).begin(); j != (*pi).end(); ++j )
	{
	    if( j != (*pi).begin() )
	    {
		m_out << " & ";
	    }
	    switch( (*j).type )
	    {
	    case ssbg::BOOL:
		output_attr( m_out, (*j).name, (*j).op, (*j).bool_val );
		break;
	    case ssbg::DOUBLE:
		output_attr( m_out, (*j).name, (*j).op, (*j).double_val );
		break;
	    case ssbg::INT:
		output_attr( m_out, (*j).name, (*j).op, (*j).int_val );
		break;
	    case ssbg::STRING:
		output_attr( m_out, (*j).name, (*j).op, (*j).string_val );
		break;
	    default:
		throw logic_error( "unknown constraint type" );
	    }
	}
    }
    m_out << " ;" << endl;
}

void SetPredicateWNP :: process_event( const ssim::Event* evt )
{
  CBNASSERT( !evt, "SetPredicateWNP can only handle NULL events" );
  ssbg::predicate pred;
  m_ssbg.new_predicate( pred );
  print( pred );
  ssim::Sim::self_signal_event( NULL, ssbg::rand_poisson_delta( m_delay ) );
}
